@@ -186,6 +186,23 @@ finishing test case.`;
     ArrayPrototypePush(tests, testDef);
   }
 
+  function formatFailure(error) {
+    if (error.errors) {
+      const message = error
+        .errors
+        .map((error) =>
+          inspectArgs([error]).replace(/^(?!\s*$)/gm, " ".repeat(4))
+        )
+        .join("\n");
+
+      return {
+        failed: error.name + "\n" + message + error.stack,
+      };
+    }
+
+    return { failed: inspectArgs([error]) };
+  }
+
   function createTestFilter(filter) {
     return (def) => {
       if (filter) {
@@ -213,10 +230,11 @@ finishing test case.`;
 
     try {
       await fn();
-      return "ok";
     } catch (error) {
-      return { "failed": inspectArgs([error]) };
+      return formatFailure(error);
     }
+
+    return "ok";
   }
 
   function getTestOrigin() {
