@@ -12,7 +12,7 @@ use deno_core::serde::Serialize;
 use deno_core::url;
 use deno_core::ModuleSpecifier;
 use deno_core::OpState;
-use log::debug;
+use log;
 use std::collections::HashSet;
 use std::fmt;
 use std::hash::Hash;
@@ -24,6 +24,10 @@ use std::sync::atomic::Ordering;
 
 const PERMISSION_EMOJI: &str = "⚠️";
 
+lazy_static::lazy_static! {
+  static ref DEBUG_LOG_ENABLED: bool = log::log_enabled!(log::Level::Debug);
+}
+
 /// Tri-state value for storing permission state
 #[derive(PartialEq, Debug, Clone, Copy, Deserialize, PartialOrd)]
 pub enum PermissionState {
@@ -35,14 +39,19 @@ pub enum PermissionState {
 impl PermissionState {
   #[inline(always)]
   fn log_perm_access(name: &str, info: Option<&str>) {
-    debug!(
-      "{}",
-      colors::bold(&format!(
-        "{}️  Granted {}",
-        PERMISSION_EMOJI,
-        Self::fmt_access(name, info)
-      ))
-    );
+    // Eliminates log overhead (when logging is disabled),
+    // log_enabled!(Debug) check in a hot path still has overhead
+    // TODO(AaronO): generalize or upstream this optimization
+    if *DEBUG_LOG_ENABLED {
+      log::debug!(
+        "{}",
+        colors::bold(&format!(
+          "{}️  Granted {}",
+          PERMISSION_EMOJI,
+          Self::fmt_access(name, info)
+        ))
+      );
+    }
   }
 
   fn fmt_access(name: &str, info: Option<&str>) -> String {
@@ -150,26 +159,48 @@ impl UnitPermission {
   }
 }
 
-#[derive(Clone, Debug, Default, Deserialize, PartialEq)]
+/// A normalized environment variable name. On Windows this will
+/// be uppercase and on other platforms it will stay as-is.
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
+struct EnvVarName {
+  inner: String,
+}
+
+impl EnvVarName {
+  pub fn new(env: impl AsRef<str>) -> Self {
+    Self {
+      inner: if cfg!(windows) {
+        env.as_ref().to_uppercase()
+      } else {
+        env.as_ref().to_string()
+      },
+    }
+  }
+}
+
+impl AsRef<str> for EnvVarName {
+  fn as_ref(&self) -> &str {
+    self.inner.as_str()
+  }
+}
+
+#[derive(Clone, Debug, Default, PartialEq)]
 pub struct UnaryPermission<T: Eq + Hash> {
-  #[serde(skip)]
   pub name: &'static str,
-  #[serde(skip)]
   pub description: &'static str,
   pub global_state: PermissionState,
   pub granted_list: HashSet<T>,
   pub denied_list: HashSet<T>,
-  #[serde(skip)]
   pub prompt: bool,
 }
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
 pub struct ReadDescriptor(pub PathBuf);
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
 pub struct WriteDescriptor(pub PathBuf);
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
 pub struct NetDescriptor(pub String, pub Option<u16>);
 
 impl NetDescriptor {
@@ -194,13 +225,25 @@ impl fmt::Display for NetDescriptor {
   }
 }
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
-pub struct EnvDescriptor(pub String);
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
+pub struct EnvDescriptor(EnvVarName);
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
+impl EnvDescriptor {
+  pub fn new(env: impl AsRef<str>) -> Self {
+    Self(EnvVarName::new(env))
+  }
+}
+
+impl AsRef<str> for EnvDescriptor {
+  fn as_ref(&self) -> &str {
+    self.0.as_ref()
+  }
+}
+
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
 pub struct RunDescriptor(pub String);
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug, Default, Deserialize)]
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
 pub struct FfiDescriptor(pub String);
 
 impl UnaryPermission<ReadDescriptor> {
@@ -240,19 +283,16 @@ impl UnaryPermission<ReadDescriptor> {
           "read access to \"{}\"",
           display_path.display()
         )) {
-          self
-            .granted_list
-            .retain(|path| !path.0.starts_with(&resolved_path));
           self.granted_list.insert(ReadDescriptor(resolved_path));
           PermissionState::Granted
         } else {
-          self
-            .denied_list
-            .retain(|path| !resolved_path.starts_with(&path.0));
           self.denied_list.insert(ReadDescriptor(resolved_path));
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(ReadDescriptor(resolved_path));
+        PermissionState::Granted
       } else {
         state
       }
@@ -278,12 +318,12 @@ impl UnaryPermission<ReadDescriptor> {
       let path = resolve_from_cwd(path).unwrap();
       self
         .granted_list
-        .retain(|path_| !path_.0.starts_with(&path));
+        .retain(|path_| !path.starts_with(&path_.0));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(path)
   }
@@ -368,19 +408,16 @@ impl UnaryPermission<WriteDescriptor> {
           "write access to \"{}\"",
           display_path.display()
         )) {
-          self
-            .granted_list
-            .retain(|path| !path.0.starts_with(&resolved_path));
           self.granted_list.insert(WriteDescriptor(resolved_path));
           PermissionState::Granted
         } else {
-          self
-            .denied_list
-            .retain(|path| !resolved_path.starts_with(&path.0));
           self.denied_list.insert(WriteDescriptor(resolved_path));
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(WriteDescriptor(resolved_path));
+        PermissionState::Granted
       } else {
         state
       }
@@ -406,12 +443,12 @@ impl UnaryPermission<WriteDescriptor> {
       let path = resolve_from_cwd(path).unwrap();
       self
         .granted_list
-        .retain(|path_| !path_.0.starts_with(&path));
+        .retain(|path_| !path.starts_with(&path_.0));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(path)
   }
@@ -477,22 +514,19 @@ impl UnaryPermission<NetDescriptor> {
   ) -> PermissionState {
     if let Some(host) = host {
       let state = self.query(Some(host));
+      let host = NetDescriptor::new(&host);
       if state == PermissionState::Prompt {
-        let host = NetDescriptor::new(&host);
         if permission_prompt(&format!("network access to \"{}\"", host)) {
-          if host.1.is_none() {
-            self.granted_list.retain(|h| h.0 != host.0);
-          }
           self.granted_list.insert(host);
           PermissionState::Granted
         } else {
-          if host.1.is_some() {
-            self.denied_list.remove(&host);
-          }
           self.denied_list.insert(host);
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(host);
+        PermissionState::Granted
       } else {
         state
       }
@@ -518,15 +552,19 @@ impl UnaryPermission<NetDescriptor> {
     host: Option<&(T, Option<u16>)>,
   ) -> PermissionState {
     if let Some(host) = host {
-      self.granted_list.remove(&NetDescriptor::new(&host));
-      if host.1.is_none() {
-        self.granted_list.retain(|h| h.0 != host.0.as_ref());
+      if host.1.is_some() {
+        self
+          .granted_list
+          .remove(&NetDescriptor(host.0.as_ref().to_string(), host.1));
       }
+      self
+        .granted_list
+        .remove(&NetDescriptor(host.0.as_ref().to_string(), None));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(host)
   }
@@ -581,21 +619,18 @@ impl UnaryPermission<NetDescriptor> {
 
 impl UnaryPermission<EnvDescriptor> {
   pub fn query(&self, env: Option<&str>) -> PermissionState {
-    #[cfg(windows)]
-    let env = env.map(|env| env.to_uppercase());
-    #[cfg(windows)]
-    let env = env.as_deref();
+    let env = env.map(EnvVarName::new);
     if self.global_state == PermissionState::Denied
-      && match env {
+      && match env.as_ref() {
         None => true,
-        Some(env) => self.denied_list.iter().any(|env_| env_.0 == env),
+        Some(env) => self.denied_list.iter().any(|env_| &env_.0 == env),
       }
     {
       PermissionState::Denied
     } else if self.global_state == PermissionState::Granted
-      || match env {
+      || match env.as_ref() {
         None => false,
-        Some(env) => self.granted_list.iter().any(|env_| env_.0 == env),
+        Some(env) => self.granted_list.iter().any(|env_| &env_.0 == env),
       }
     {
       PermissionState::Granted
@@ -606,23 +641,19 @@ impl UnaryPermission<EnvDescriptor> {
 
   pub fn request(&mut self, env: Option<&str>) -> PermissionState {
     if let Some(env) = env {
-      let env = if cfg!(windows) {
-        env.to_uppercase()
-      } else {
-        env.to_string()
-      };
-      let state = self.query(Some(&env));
+      let state = self.query(Some(env));
       if state == PermissionState::Prompt {
         if permission_prompt(&format!("env access to \"{}\"", env)) {
-          self.granted_list.retain(|env_| env_.0 != env);
-          self.granted_list.insert(EnvDescriptor(env));
+          self.granted_list.insert(EnvDescriptor::new(env));
           PermissionState::Granted
         } else {
-          self.denied_list.retain(|env_| env_.0 != env);
-          self.denied_list.insert(EnvDescriptor(env));
+          self.denied_list.insert(EnvDescriptor::new(env));
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(EnvDescriptor::new(env));
+        PermissionState::Granted
       } else {
         state
       }
@@ -645,21 +676,17 @@ impl UnaryPermission<EnvDescriptor> {
 
   pub fn revoke(&mut self, env: Option<&str>) -> PermissionState {
     if let Some(env) = env {
-      #[cfg(windows)]
-      let env = env.to_uppercase();
-      self.granted_list.retain(|env_| env_.0 != env);
+      self.granted_list.remove(&EnvDescriptor::new(env));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(env)
   }
 
   pub fn check(&mut self, env: &str) -> Result<(), AnyError> {
-    #[cfg(windows)]
-    let env = &env.to_uppercase();
     let (result, prompted) = self.query(Some(env)).check(
       self.name,
       Some(&format!("\"{}\"", env)),
@@ -667,9 +694,9 @@ impl UnaryPermission<EnvDescriptor> {
     );
     if prompted {
       if result.is_ok() {
-        self.granted_list.insert(EnvDescriptor(env.to_string()));
+        self.granted_list.insert(EnvDescriptor::new(env));
       } else {
-        self.denied_list.insert(EnvDescriptor(env.to_string()));
+        self.denied_list.insert(EnvDescriptor::new(env));
         self.global_state = PermissionState::Denied;
       }
     }
@@ -716,15 +743,16 @@ impl UnaryPermission<RunDescriptor> {
       let state = self.query(Some(cmd));
       if state == PermissionState::Prompt {
         if permission_prompt(&format!("run access to \"{}\"", cmd)) {
-          self.granted_list.retain(|cmd_| cmd_.0 != cmd);
           self.granted_list.insert(RunDescriptor(cmd.to_string()));
           PermissionState::Granted
         } else {
-          self.denied_list.retain(|cmd_| cmd_.0 != cmd);
           self.denied_list.insert(RunDescriptor(cmd.to_string()));
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(RunDescriptor(cmd.to_string()));
+        PermissionState::Granted
       } else {
         state
       }
@@ -747,12 +775,12 @@ impl UnaryPermission<RunDescriptor> {
 
   pub fn revoke(&mut self, cmd: Option<&str>) -> PermissionState {
     if let Some(cmd) = cmd {
-      self.granted_list.retain(|cmd_| cmd_.0 != cmd);
+      self.granted_list.remove(&RunDescriptor(cmd.to_string()));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(cmd)
   }
@@ -814,15 +842,16 @@ impl UnaryPermission<FfiDescriptor> {
       let state = self.query(Some(lib));
       if state == PermissionState::Prompt {
         if permission_prompt(&format!("ffi access to \"{}\"", lib)) {
-          self.granted_list.retain(|lib_| lib_.0 != lib);
           self.granted_list.insert(FfiDescriptor(lib.to_string()));
           PermissionState::Granted
         } else {
-          self.denied_list.retain(|lib_| lib_.0 != lib);
           self.denied_list.insert(FfiDescriptor(lib.to_string()));
           self.global_state = PermissionState::Denied;
           PermissionState::Denied
         }
+      } else if state == PermissionState::Granted {
+        self.granted_list.insert(FfiDescriptor(lib.to_string()));
+        PermissionState::Granted
       } else {
         state
       }
@@ -845,12 +874,12 @@ impl UnaryPermission<FfiDescriptor> {
 
   pub fn revoke(&mut self, lib: Option<&str>) -> PermissionState {
     if let Some(lib) = lib {
-      self.granted_list.retain(|lib_| lib_.0 != lib);
+      self.granted_list.remove(&FfiDescriptor(lib.to_string()));
     } else {
       self.granted_list.clear();
-      if self.global_state == PermissionState::Granted {
-        self.global_state = PermissionState::Prompt;
-      }
+    }
+    if self.global_state == PermissionState::Granted {
+      self.global_state = PermissionState::Prompt;
     }
     self.query(lib)
   }
@@ -969,17 +998,7 @@ impl Permissions {
       global_state: global_state_from_option(state),
       granted_list: state
         .as_ref()
-        .map(|v| {
-          v.iter()
-            .map(|x| {
-              EnvDescriptor(if cfg!(windows) {
-                x.to_uppercase()
-              } else {
-                x.clone()
-              })
-            })
-            .collect()
-        })
+        .map(|v| v.iter().map(EnvDescriptor::new).collect())
         .unwrap_or_else(HashSet::new),
       denied_list: Default::default(),
       prompt,
@@ -1769,15 +1788,24 @@ mod tests {
     let mut perms = Permissions {
       read: UnaryPermission {
         global_state: PermissionState::Prompt,
-        ..Permissions::new_read(&Some(vec![PathBuf::from("/foo")]), false)
+        ..Permissions::new_read(
+          &Some(vec![PathBuf::from("/foo"), PathBuf::from("/foo/baz")]),
+          false,
+        )
       },
       write: UnaryPermission {
         global_state: PermissionState::Prompt,
-        ..Permissions::new_write(&Some(vec![PathBuf::from("/foo")]), false)
+        ..Permissions::new_write(
+          &Some(vec![PathBuf::from("/foo"), PathBuf::from("/foo/baz")]),
+          false,
+        )
       },
       net: UnaryPermission {
         global_state: PermissionState::Prompt,
-        ..Permissions::new_net(&Some(svec!["127.0.0.1"]), false)
+        ..Permissions::new_net(
+          &Some(svec!["127.0.0.1", "127.0.0.1:8000"]),
+          false,
+        )
       },
       env: UnaryPermission {
         global_state: PermissionState::Prompt,
@@ -1798,14 +1826,15 @@ mod tests {
     };
     #[rustfmt::skip]
     {
-      assert_eq!(perms.read.revoke(Some(Path::new("/foo/bar"))), PermissionState::Granted);
-      assert_eq!(perms.read.revoke(Some(Path::new("/foo"))), PermissionState::Prompt);
-      assert_eq!(perms.read.query(Some(Path::new("/foo/bar"))), PermissionState::Prompt);
-      assert_eq!(perms.write.revoke(Some(Path::new("/foo/bar"))), PermissionState::Granted);
-      assert_eq!(perms.write.revoke(None), PermissionState::Prompt);
-      assert_eq!(perms.write.query(Some(Path::new("/foo/bar"))), PermissionState::Prompt);
-      assert_eq!(perms.net.revoke(Some(&("127.0.0.1", Some(8000)))), PermissionState::Granted);
-      assert_eq!(perms.net.revoke(Some(&("127.0.0.1", None))), PermissionState::Prompt);
+      assert_eq!(perms.read.revoke(Some(Path::new("/foo/bar"))), PermissionState::Prompt);
+      assert_eq!(perms.read.query(Some(Path::new("/foo"))), PermissionState::Prompt);
+      assert_eq!(perms.read.query(Some(Path::new("/foo/baz"))), PermissionState::Granted);
+      assert_eq!(perms.write.revoke(Some(Path::new("/foo/bar"))), PermissionState::Prompt);
+      assert_eq!(perms.write.query(Some(Path::new("/foo"))), PermissionState::Prompt);
+      assert_eq!(perms.write.query(Some(Path::new("/foo/baz"))), PermissionState::Granted);
+      assert_eq!(perms.net.revoke(Some(&("127.0.0.1", Some(9000)))), PermissionState::Prompt);
+      assert_eq!(perms.net.query(Some(&("127.0.0.1", None))), PermissionState::Prompt);
+      assert_eq!(perms.net.query(Some(&("127.0.0.1", Some(8000)))), PermissionState::Granted);
       assert_eq!(perms.env.revoke(Some(&"HOME".to_string())), PermissionState::Prompt);
       assert_eq!(perms.run.revoke(Some(&"deno".to_string())), PermissionState::Prompt);
       assert_eq!(perms.ffi.revoke(Some(&"deno".to_string())), PermissionState::Prompt);
