@@ -13,12 +13,3 @@ pub fn create_basic_runtime() -> tokio::runtime::Runtime {
     .build()
     .unwrap()
 }
-
-// TODO(ry) rename to run_local ?
-pub fn run_basic<F, R>(future: F) -> R
-where
-  F: std::future::Future<Output = R>,
-{
-  let rt = create_basic_runtime();
-  rt.block_on(future)
-}
